@@ -15,6 +15,8 @@ Configuration::Configuration()
 	instSSGH_ = 390;
 	followMode_ = true;
 	workDir_ = "";
+	instOpenFormat_ = 0;
+	bankOpenFormat_ = 0;
 
 	// General //
 	// General settings
@@ -30,6 +32,7 @@ Configuration::Configuration()
 	enableTranslation_ = true;
 	showFMDetuneSigned_ = false;
 	showWaveVisual_ = true;
+	fill00ToEffectValue_ = true;
 
 	// Edit settings
 	pageJumpLength_ = 4;
@@ -694,6 +697,26 @@ std::string Configuration::getWorkingDirectory() const
 	return workDir_;
 }
 
+void Configuration::setInstrumentOpenFormat(int i)
+{
+	instOpenFormat_ = i;
+}
+
+int Configuration::getInstrumentOpenFormat() const
+{
+	return instOpenFormat_;
+}
+
+void Configuration::setBankOpenFormat(int i)
+{
+	bankOpenFormat_ = i;
+}
+
+int Configuration::getBankOpenFormat() const
+{
+	return bankOpenFormat_;
+}
+
 // General //
 // General settings
 void Configuration::setWarpCursor(bool enabled)
@@ -815,6 +838,16 @@ bool Configuration::getShowWaveVisual() const
 	return showWaveVisual_;
 }
 
+void Configuration::setFill00ToEffectValue(bool enabled)
+{
+	fill00ToEffectValue_ = enabled;
+}
+
+bool Configuration::getFill00ToEffectValue() const
+{
+	return fill00ToEffectValue_;
+}
+
 // Edit settings
 void Configuration::setPageJumpLength(size_t length)
 {
