@@ -44,6 +44,10 @@ public:
 	bool getFollowMode() const;
 	void setWorkingDirectory(std::string path);
 	std::string getWorkingDirectory() const;
+	void setInstrumentOpenFormat(int i);
+	int getInstrumentOpenFormat() const;
+	void setBankOpenFormat(int i);
+	int getBankOpenFormat() const;
 private:
 	int mainW_, mainH_;
 	bool mainMax_;
@@ -52,6 +56,7 @@ private:
 	int instSSGW_, instSSGH_;
 	bool followMode_;
 	std::string workDir_;
+	int instOpenFormat_, bankOpenFormat_;
 
 	// General //
 	// General settings
@@ -80,6 +85,8 @@ public:
 	bool getShowFMDetuneAsSigned() const;
 	void setShowWaveVisual(bool enabled);
 	bool getShowWaveVisual() const;
+	void setFill00ToEffectValue(bool enabled);
+	bool getFill00ToEffectValue() const;
 private:
 	bool warpCursor_, warpAcrossOrders_;
 	bool showRowNumHex_, showPrevNextOrders_;
@@ -88,6 +95,7 @@ private:
 	bool retrieveChannelState_, enableTranslation_;
 	bool showFMDetuneSigned_;
 	bool showWaveVisual_;
+	bool fill00ToEffectValue_;
 
 	// Edit settings
 public:
