@@ -95,7 +95,7 @@ void FMOperatorTable::setColorPalette(std::shared_ptr<ColorPalette> palette)
 void FMOperatorTable::setOperatorNumber(int n)
 {
 	number_ = n;
-	ui->groupBox->setTitle("Operator " + QString::number(n + 1));
+	ui->groupBox->setTitle(tr("Operator ") + QString::number(n + 1));
 }
 
 int FMOperatorTable::operatorNumber() const
