@@ -2,7 +2,10 @@
 #include "ui_instrument_editor_ssg_form.h"
 #include <vector>
 #include <utility>
+#include <stdexcept>
 #include "gui/event_guard.hpp"
+#include "pitch_converter.hpp"
+#include "command_sequence.hpp"
 #include "misc.hpp"
 
 InstrumentEditorSSGForm::InstrumentEditorSSGForm(int num, QWidget *parent) :
@@ -13,31 +16,21 @@ InstrumentEditorSSGForm::InstrumentEditorSSGForm(int num, QWidget *parent) :
 	ui->setupUi(this);
 
 	//========== Wave form ==========//
-	ui->waveEditor->setMaximumDisplayedRowCount(5);
+	ui->waveEditor->setMaximumDisplayedRowCount(7);
 	ui->waveEditor->setDefaultRow(0);
-	ui->waveEditor->AddRow("Sq");
-	ui->waveEditor->AddRow("Tri");
-	ui->waveEditor->AddRow("Saw");
-	ui->waveEditor->AddRow("Tri w");
-	ui->waveEditor->AddRow("Saw w");
-
-	QString tn[] = { "C", "C#", "D", "D#", "E", "F", "F#", "G", "G#", "A", "A#", "B" };
-	for (int i = 0; i < 8; ++i) {
-		for (int j = 0; j < 12; ++j) {
-			for (int k = 0; k < 32; ++k) {
-				ui->squareMaskComboBox->addItem(
-							QString("%1%2+%3").arg(tn[j]).arg(i).arg(k), 12 * i + 32 * j + k);
-			}
-		}
-	}
+	ui->waveEditor->AddRow("Sq", false);
+	ui->waveEditor->AddRow("Tri", false);
+	ui->waveEditor->AddRow("Saw", false);
+	ui->waveEditor->AddRow("InvSaw", false);
+	ui->waveEditor->AddRow("SMTri", false);
+	ui->waveEditor->AddRow("SMSaw", false);
+	ui->waveEditor->AddRow("SMInvSaw", false);
+	ui->waveEditor->autoFitLabelWidth();
 
 	QObject::connect(ui->waveEditor, &VisualizedInstrumentMacroEditor::sequenceCommandAdded,
 					 this, [&](int row, int col) {
 		if (!isIgnoreEvent_) {
-			if (row >= 3) {	// Set square mask frequency
-				ui->waveEditor->setText(col, ui->squareMaskComboBox->currentText());
-				ui->waveEditor->setData(col, ui->squareMaskComboBox->currentData().toInt());
-			}
+			if (isModulatedWaveFormSSG(row)) setWaveFormSequenceColumn(col);	// Set square-mask frequency
 			bt_.lock()->addWaveFormSSGSequenceCommand(
 						ui->waveNumSpinBox->value(), row, ui->waveEditor->getSequenceDataAt(col));
 			emit waveFormParameterChanged(ui->waveNumSpinBox->value(), instNum_);
@@ -55,10 +48,7 @@ InstrumentEditorSSGForm::InstrumentEditorSSGForm(int num, QWidget *parent) :
 	QObject::connect(ui->waveEditor, &VisualizedInstrumentMacroEditor::sequenceCommandChanged,
 					 this, [&](int row, int col) {
 		if (!isIgnoreEvent_) {
-			if (row >= 3) {	// Set square mask frequency
-				ui->waveEditor->setText(col, ui->squareMaskComboBox->currentText());
-				ui->waveEditor->setData(col, ui->squareMaskComboBox->currentData().toInt());
-			}
+			if (isModulatedWaveFormSSG(row)) setWaveFormSequenceColumn(col);	// Set square-mask frequency
 			bt_.lock()->setWaveFormSSGSequenceCommand(
 						ui->waveNumSpinBox->value(), col, row, ui->waveEditor->getSequenceDataAt(col));
 			emit waveFormParameterChanged(ui->waveNumSpinBox->value(), instNum_);
@@ -87,13 +77,14 @@ InstrumentEditorSSGForm::InstrumentEditorSSGForm(int num, QWidget *parent) :
 	//========== Tone/Noise ==========//
 	ui->tnEditor->setMaximumDisplayedRowCount(16);
 	ui->tnEditor->setDefaultRow(0);
-	ui->tnEditor->AddRow("T");
+	ui->tnEditor->AddRow("T", false);
 	for (int i = 0; i < 32; ++i) {
-		ui->tnEditor->AddRow("N " + QString::number(i));
+		ui->tnEditor->AddRow("N " + QString::number(i), false);
 	}
 	for (int i = 0; i < 32; ++i) {
-		ui->tnEditor->AddRow("T&N " + QString::number(i));
+		ui->tnEditor->AddRow("T&N " + QString::number(i), false);
 	}
+	ui->tnEditor->autoFitLabelWidth();
 
 	QObject::connect(ui->tnEditor, &VisualizedInstrumentMacroEditor::sequenceCommandAdded,
 					 this, [&](int row, int col) {
@@ -144,23 +135,18 @@ InstrumentEditorSSGForm::InstrumentEditorSSGForm(int num, QWidget *parent) :
 	ui->envEditor->setMaximumDisplayedRowCount(16);
 	ui->envEditor->setDefaultRow(15);
 	for (int i = 0; i < 16; ++i) {
-		ui->envEditor->AddRow(QString::number(i));
+		ui->envEditor->AddRow(QString::number(i), false);
 	}
 	for (int i = 0; i < 8; ++i) {
-		ui->envEditor->AddRow("Hard " + QString::number(i));
+		ui->envEditor->AddRow("Hard " + QString::number(i), false);
 	}
+	ui->envEditor->autoFitLabelWidth();
 	ui->envEditor->setMultipleReleaseState(true);
 
-	ui->hardFreqSpinBox->setSuffix(
-				QString(" (%1Hz)").arg(QString::number(7800.0 / ui->hardFreqSpinBox->value(), 'f', 4)));
-
 	QObject::connect(ui->envEditor, &VisualizedInstrumentMacroEditor::sequenceCommandAdded,
 					 this, [&](int row, int col) {
 		if (!isIgnoreEvent_) {
-			if (row >= 16) {	// Set hard frequency
-				ui->envEditor->setText(col, QString::number(ui->hardFreqSpinBox->value()));
-				ui->envEditor->setData(col, ui->hardFreqSpinBox->value());
-			}
+			if (row >= 16) setEnvelopeSequenceColumn(col);	// Set hard frequency
 			bt_.lock()->addEnvelopeSSGSequenceCommand(
 						ui->envNumSpinBox->value(), row, ui->envEditor->getSequenceDataAt(col));
 			emit envelopeParameterChanged(ui->envNumSpinBox->value(), instNum_);
@@ -178,10 +164,7 @@ InstrumentEditorSSGForm::InstrumentEditorSSGForm(int num, QWidget *parent) :
 	QObject::connect(ui->envEditor, &VisualizedInstrumentMacroEditor::sequenceCommandChanged,
 					 this, [&](int row, int col) {
 		if (!isIgnoreEvent_) {
-			if (row >= 16) {	// Set hard frequency
-				ui->envEditor->setText(col, QString::number(ui->hardFreqSpinBox->value()));
-				ui->envEditor->setData(col, ui->hardFreqSpinBox->value());
-			}
+			if (row >= 16) setEnvelopeSequenceColumn(col);	// Set hard frequency
 			bt_.lock()->setEnvelopeSSGSequenceCommand(
 						ui->envNumSpinBox->value(), col, row, ui->envEditor->getSequenceDataAt(col));
 			emit envelopeParameterChanged(ui->envNumSpinBox->value(), instNum_);
@@ -215,8 +198,9 @@ InstrumentEditorSSGForm::InstrumentEditorSSGForm(int num, QWidget *parent) :
 		int d = i - 48;
 		auto text = QString::number(d);
 		if (d > 0) text = "+" + text;
-		ui->arpEditor->AddRow(text);
+		ui->arpEditor->AddRow(text, false);
 	}
+	ui->arpEditor->autoFitLabelWidth();
 	ui->arpEditor->setUpperRow(55);
 	ui->arpEditor->setMMLDisplay0As(-48);
 
@@ -268,7 +252,7 @@ InstrumentEditorSSGForm::InstrumentEditorSSGForm(int num, QWidget *parent) :
 			emit modified();
 		}
 	});
-// Leave Before Qt5.7.0 style due to windows xp
+	// Leave Before Qt5.7.0 style due to windows xp
 	QObject::connect(ui->arpTypeComboBox, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
 					 this, &InstrumentEditorSSGForm::onArpeggioTypeChanged);
 
@@ -280,8 +264,9 @@ InstrumentEditorSSGForm::InstrumentEditorSSGForm(int num, QWidget *parent) :
 		int d = i - 127;
 		auto text = QString::number(d);
 		if (d > 0) text = "+" + text;
-		ui->ptEditor->AddRow(text);
+		ui->ptEditor->AddRow(text, false);
 	}
+	ui->ptEditor->autoFitLabelWidth();
 	ui->ptEditor->setUpperRow(134);
 	ui->ptEditor->setMMLDisplay0As(-127);
 
@@ -332,7 +317,7 @@ InstrumentEditorSSGForm::InstrumentEditorSSGForm(int num, QWidget *parent) :
 			emit modified();
 		}
 	});
-// Leave Before Qt5.7.0 style due to windows xp
+	// Leave Before Qt5.7.0 style due to windows xp
 	QObject::connect(ui->ptTypeComboBox, static_cast<void(QComboBox::*)(int)>(&QComboBox::currentIndexChanged),
 					 this, &InstrumentEditorSSGForm::onPitchTypeChanged);
 }
@@ -405,7 +390,7 @@ void InstrumentEditorSSGForm::updateInstrumentParameters()
 
 	std::unique_ptr<AbstractInstrument> inst = bt_.lock()->getInstrument(instNum_);
 	auto instSSG = dynamic_cast<InstrumentSSG*>(inst.get());
-	auto name = QString::fromUtf8(instSSG->getName().c_str(), instSSG->getName().length());
+	auto name = QString::fromUtf8(instSSG->getName().c_str(), static_cast<int>(instSSG->getName().length()));
 	setWindowTitle(QString("%1: %2").arg(instNum_, 2, 16, QChar('0')).toUpper().arg(name));
 
 	setInstrumentWaveFormParameters();
@@ -422,14 +407,16 @@ void InstrumentEditorSSGForm::keyPressEvent(QKeyEvent *event)
 	// For jam key on
 
 	// Check keys
-	QString seq = QKeySequence(event->modifiers() | event->key()).toString();
-	if (seq == QKeySequence(QString::fromUtf8(config_.lock()->getOctaveUpKey().c_str(),
-											  config_.lock()->getOctaveUpKey().length())).toString()) {
+	QString seq = QKeySequence(static_cast<int>(event->modifiers()) | event->key()).toString();
+	if (seq == QKeySequence(
+				QString::fromUtf8(config_.lock()->getOctaveUpKey().c_str(),
+								  static_cast<int>(config_.lock()->getOctaveUpKey().length()))).toString()) {
 		emit octaveChanged(true);
 		return;
 	}
-	else if (seq == QKeySequence(QString::fromUtf8(config_.lock()->getOctaveDownKey().c_str(),
-												   config_.lock()->getOctaveDownKey().length())).toString()) {
+	else if (seq == QKeySequence(
+				 QString::fromUtf8(config_.lock()->getOctaveDownKey().c_str(),
+								   static_cast<int>(config_.lock()->getOctaveDownKey().length()))).toString()) {
 		emit octaveChanged(false);
 		return;
 	}
@@ -539,23 +526,15 @@ void InstrumentEditorSSGForm::setInstrumentWaveFormParameters()
 	ui->waveNumSpinBox->setValue(instSSG->getWaveFormNumber());
 	ui->waveEditor->clearData();
 	for (auto& com : instSSG->getWaveFormSequence()) {
-		QString str = "";
-		if (com.type >= 3) {
-			switch (com.data / 32 % 12) {
-				case 0:		str = "C";	break;
-				case 1:		str = "C#";	break;
-				case 2:		str = "D";	break;
-				case 3:		str = "D#";	break;
-				case 4:		str = "E";	break;
-				case 5:		str = "F";	break;
-				case 6:		str = "F#";	break;
-				case 7:		str = "G";	break;
-				case 8:		str = "G#";	break;
-				case 9:		str = "A";	break;
-				case 10:	str = "A#";	break;
-				case 11:	str = "B";	break;
+		QString str("");
+		if (isModulatedWaveFormSSG(com.type)) {
+			if (CommandSequenceUnit::isRatioData(com.data)) {
+				auto ratio = CommandSequenceUnit::data2ratio(com.data);
+				str = QString::number(ratio.first) + "/" + QString::number(ratio.second);
+			}
+			else {
+				str = QString::number(com.data);
 			}
-			str += QString("%1+%2").arg(com.data / 32 / 12).arg(com.data % 32);
 		}
 		ui->waveEditor->addSequenceCommand(com.type, str, com.data);
 	}
@@ -573,6 +552,23 @@ void InstrumentEditorSSGForm::setInstrumentWaveFormParameters()
 	}
 }
 
+void InstrumentEditorSSGForm::setWaveFormSequenceColumn(int col)
+{
+	auto button = ui->squareMaskButtonGroup->checkedButton();
+	if (button == ui->squareMaskRawRadioButton) {
+		ui->waveEditor->setText(col, QString::number(ui->squareMaskRawSpinBox->value()));
+		ui->waveEditor->setData(col, ui->squareMaskRawSpinBox->value());
+	}
+	else {
+		ui->waveEditor->setText(col, QString::number(ui->squareMaskToneSpinBox->value()) + "/"
+								+ QString::number(ui->squareMaskMaskSpinBox->value()));
+
+		ui->waveEditor->setData(col, CommandSequenceUnit::ratio2data(
+									ui->squareMaskToneSpinBox->value(),
+									ui->squareMaskMaskSpinBox->value()));
+	}
+}
+
 /********** Slots **********/
 void InstrumentEditorSSGForm::onWaveFormNumberChanged()
 {
@@ -619,6 +615,14 @@ void InstrumentEditorSSGForm::on_waveNumSpinBox_valueChanged(int arg1)
 	onWaveFormNumberChanged();
 }
 
+void InstrumentEditorSSGForm::on_squareMaskRawSpinBox_valueChanged(int arg1)
+{
+	ui->squareMaskRawSpinBox->setSuffix(
+				QString(" (0x") + QString("%1 | ").arg(arg1, 3, 16, QChar('0')).toUpper()
+				+ QString("%1Hz)").arg(arg1 ? QString::number(124800.0 / arg1, 'f', 4) : "-")
+				);
+}
+
 //--- Tone/Noise
 void InstrumentEditorSSGForm::setInstrumentToneNoiseParameters()
 {
@@ -703,9 +707,15 @@ void InstrumentEditorSSGForm::setInstrumentEnvelopeParameters()
 	ui->envNumSpinBox->setValue(instSSG->getEnvelopeNumber());
 	ui->envEditor->clearData();
 	for (auto& com : instSSG->getEnvelopeSequence()) {
-		QString str = "";
+		QString str("");
 		if (com.type >= 16) {
-			str = QString("%1").arg(com.data);
+			if (CommandSequenceUnit::isRatioData(com.data)) {
+				auto ratio = CommandSequenceUnit::data2ratio(com.data);
+				str = QString::number(ratio.first) + "/" + QString::number(ratio.second);
+			}
+			else {
+				str = QString::number(com.data);
+			}
 		}
 		ui->envEditor->addSequenceCommand(com.type, str, com.data);
 	}
@@ -723,6 +733,22 @@ void InstrumentEditorSSGForm::setInstrumentEnvelopeParameters()
 	}
 }
 
+void InstrumentEditorSSGForm::setEnvelopeSequenceColumn(int col)
+{
+	if (ui->hardFreqButtonGroup->checkedButton() == ui->hardFreqRawRadioButton) {
+		ui->envEditor->setText(col, QString::number(ui->hardFreqRawSpinBox->value()));
+		ui->envEditor->setData(col, ui->hardFreqRawSpinBox->value());
+	}
+	else {
+		ui->envEditor->setText(col, QString::number(ui->hardFreqToneSpinBox->value()) + "/"
+							   + QString::number(ui->hardFreqHardSpinBox->value()));
+
+		ui->envEditor->setData(col, CommandSequenceUnit::ratio2data(
+								   ui->hardFreqToneSpinBox->value(),
+								   ui->hardFreqHardSpinBox->value()));
+	}
+}
+
 /********** Slots **********/
 void InstrumentEditorSSGForm::onEnvelopeNumberChanged()
 {
@@ -769,12 +795,11 @@ void InstrumentEditorSSGForm::on_envNumSpinBox_valueChanged(int arg1)
 	onEnvelopeNumberChanged();
 }
 
-void InstrumentEditorSSGForm::on_hardFreqSpinBox_valueChanged(int arg1)
+void InstrumentEditorSSGForm::on_hardFreqRawSpinBox_valueChanged(int arg1)
 {
-	Q_UNUSED(arg1)
-
-	ui->hardFreqSpinBox->setSuffix(
-				QString(" (%1Hz)").arg(QString::number(7800.0 / ui->hardFreqSpinBox->value(), 'f', 4)));
+	ui->hardFreqRawSpinBox->setSuffix(
+				QString(" (0x") + QString("%1 | ").arg(arg1, 4, 16, QChar('0')).toUpper()
+				+ QString("%1Hz").arg(arg1 ? QString::number(7800.0 / arg1, 'f', 4) : "-"));
 }
 
 //--- Arpeggio
@@ -898,7 +923,7 @@ void InstrumentEditorSSGForm::setInstrumentPitchParameters()
 		ui->ptEditor->addLoop(l.begin, l.end, l.times);
 	}
 	ui->ptEditor->setRelease(convertReleaseTypeForUI(instSSG->getPitchRelease().type),
-							  instSSG->getPitchRelease().begin);
+							 instSSG->getPitchRelease().begin);
 	for (int i = 0; i < ui->ptTypeComboBox->count(); ++i) {
 		if (ui->ptTypeComboBox->itemData(i, Qt::UserRole).toInt() == instSSG->getPitchType()) {
 			ui->ptTypeComboBox->setCurrentIndex(i);
