@@ -1,6 +1,7 @@
 #include "mainwindow.hpp"
 #include "ui_mainwindow.h"
 #include <nowide/fstream.hpp>
+#include <algorithm>
 #include <QString>
 #include <QLineEdit>
 #include <QClipboard>
@@ -22,6 +23,7 @@
 #include "instrument.hpp"
 #include "bank.hpp"
 #include "bank_io.hpp"
+#include "file_io.hpp"
 #include "version.hpp"
 #include "gui/command/commands_qt.hpp"
 #include "gui/instrument_editor/instrument_editor_fm_form.hpp"
@@ -557,39 +559,67 @@ void MainWindow::keyReleaseEvent(QKeyEvent *event)
 void MainWindow::dragEnterEvent(QDragEnterEvent* event)
 {
 	auto mime = event->mimeData();
-	if (mime->hasUrls() && mime->urls().length() == 1
-			&& mime->urls().first().toLocalFile().endsWith(".btm"))
-		event->acceptProposedAction();
-}
-
-void MainWindow::dropEvent(QDropEvent* event)
-{
-	if (isWindowModified()) {
-		auto modTitleStd = bt_->getModuleTitle();
-		QString modTitle = QString::fromUtf8(modTitleStd.c_str(), static_cast<int>(modTitleStd.length()));
-		if (modTitle.isEmpty()) modTitle = tr("Untitled");
-		QMessageBox dialog(QMessageBox::Warning,
-						   "BambooTracker",
-						   tr("Save changes to %1?").arg(modTitle),
-						   QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
-		switch (dialog.exec()) {
-		case QMessageBox::Yes:
-			if (!on_actionSave_triggered()) return;
+	if (mime->hasUrls() && mime->urls().length() == 1) {
+		switch (FileIO::judgeFileTypeFromExtension(
+					QFileInfo(mime->urls().first().toLocalFile()).suffix().toStdString())) {
+		case FileIO::FileType::MOD:
+		case FileIO::FileType::INST:
+		case FileIO::FileType::BANK:
+			event->acceptProposedAction();
 			break;
-		case QMessageBox::No:
-			break;
-		case QMessageBox::Cancel:
-			return;
 		default:
 			break;
 		}
 	}
+}
 
-	bt_->stopPlaySong();
-	lockControls(false);
-
+void MainWindow::dropEvent(QDropEvent* event)
+{
 	QString file = event->mimeData()->urls().first().toLocalFile();
-	openModule(file);
+
+	switch (FileIO::judgeFileTypeFromExtension(QFileInfo(file).suffix().toStdString())) {
+	case FileIO::FileType::MOD:
+	{
+		if (isWindowModified()) {
+			auto modTitleStd = bt_->getModuleTitle();
+			QString modTitle = QString::fromUtf8(modTitleStd.c_str(), static_cast<int>(modTitleStd.length()));
+			if (modTitle.isEmpty()) modTitle = tr("Untitled");
+			QMessageBox dialog(QMessageBox::Warning,
+							   "BambooTracker",
+							   tr("Save changes to %1?").arg(modTitle),
+							   QMessageBox::Yes | QMessageBox::No | QMessageBox::Cancel);
+			switch (dialog.exec()) {
+			case QMessageBox::Yes:
+				if (!on_actionSave_triggered()) return;
+				break;
+			case QMessageBox::No:
+				break;
+			case QMessageBox::Cancel:
+				return;
+			default:
+				break;
+			}
+		}
+
+		bt_->stopPlaySong();
+		lockControls(false);
+
+		openModule(file);
+		break;
+	}
+	case FileIO::FileType::INST:
+	{
+		funcLoadInstrument(file);
+		break;
+	}
+	case FileIO::FileType::BANK:
+	{
+		funcImportInstrumentsFromBank(file);
+		break;
+	}
+	default:
+		break;
+	}
 }
 
 void MainWindow::resizeEvent(QResizeEvent* event)
@@ -802,16 +832,32 @@ void MainWindow::deepCloneInstrument()
 void MainWindow::loadInstrument()
 {
 	QString dir = QString::fromStdString(config_.lock()->getWorkingDirectory());
-	QString file = QFileDialog::getOpenFileName(this, tr("Open instrument"), (dir.isEmpty() ? "./" : dir),
-												QStringList{tr("BambooTracker instrument (*.bti)"),
-														tr("DefleMask preset (*.dmp)"),
-														tr("TFM Music Maker instrument (*.tfi)"),
-														tr("VGM Music Maker instrument (*.vgi)"),
-														tr("WOPN instrument (*.opni)"),
-														tr("Gens KMod dump (*.y12)"),
-														tr("MVSTracker instrument (*.ins)")}.join(";;"));
-	if (file.isNull()) return;
+	QStringList filters {
+		tr("BambooTracker instrument (*.bti)"),
+		tr("DefleMask preset (*.dmp)"),
+		tr("TFM Music Maker instrument (*.tfi)"),
+		tr("VGM Music Maker instrument (*.vgi)"),
+		tr("WOPN instrument (*.opni)"),
+		tr("Gens KMod dump (*.y12)"),
+		tr("MVSTracker instrument (*.ins)")
+	};
+	QString defaultFilter = filters.at(config_.lock()->getInstrumentOpenFormat());
 
+	QString file = QFileDialog::getOpenFileName(this, tr("Open instrument"), (dir.isEmpty() ? "./" : dir),
+												filters.join(";;"), &defaultFilter);
+	if (file.isNull()) {
+		return;
+	}
+	else {
+		int index = getSelectedFileFilter(file, filters);
+		if (index != -1) config_.lock()->setInstrumentOpenFormat(index);
+	}
+
+	funcLoadInstrument(file);
+}
+
+void MainWindow::funcLoadInstrument(QString file)
+{
 	int n = bt_->findFirstFreeInstrumentNumber();
 	if (n == -1) QMessageBox::critical(this, tr("Error"), tr("Failed to load instrument."));
 
@@ -823,6 +869,7 @@ void MainWindow::loadInstrument()
 							ui->instrumentListWidget, n,
 							QString::fromUtf8(name.c_str(), static_cast<int>(name.length())),
 							inst->getSoundSource(), instForms_));
+		ui->instrumentListWidget->setCurrentRow(n);
 		config_.lock()->setWorkingDirectory(QFileInfo(file).dir().path().toStdString());
 	}
 	catch (std::exception& e) {
@@ -856,10 +903,27 @@ void MainWindow::saveInstrument()
 void MainWindow::importInstrumentsFromBank()
 {
 	QString dir = QString::fromStdString(config_.lock()->getWorkingDirectory());
-	QString file = QFileDialog::getOpenFileName(this, tr("Open bank"), (dir.isEmpty() ? "./" : dir),
-												tr("WOPN bank (*.wopn)"));
-	if (file.isNull()) return;
+	QStringList filters {
+		tr("BambooTracker bank (*.btb)"),
+		tr("WOPN bank (*.wopn)")
+	};
+	QString defaultFilter = filters.at(config_.lock()->getBankOpenFormat());
 
+	QString file = QFileDialog::getOpenFileName(this, tr("Open bank"), (dir.isEmpty() ? "./" : dir),
+												filters.join(";;"), &defaultFilter);
+	if (file.isNull()) {
+		return;
+	}
+	else {
+		int index = getSelectedFileFilter(file, filters);
+		if (index != -1) config_.lock()->setBankOpenFormat(index);
+	}
+
+	funcImportInstrumentsFromBank(file);
+}
+
+void MainWindow::funcImportInstrumentsFromBank(QString file)
+{
 	std::unique_ptr<AbstractBank> bank;
 	try {
 		bank.reset(BankIO::loadBank(file.toStdString()));
@@ -875,12 +939,15 @@ void MainWindow::importInstrumentsFromBank()
 		return;
 
 	QVector<size_t> selection = dlg.currentInstrumentSelection();
+	if (selection.empty()) return;
 
 	try {
+		int lastNum = ui->instrumentListWidget->currentRow();
 		for (size_t index : selection) {
 			int n = bt_->findFirstFreeInstrumentNumber();
 			if (n == -1){
 				QMessageBox::critical(this, tr("Error"), tr("Failed to load instrument."));
+				ui->instrumentListWidget->setCurrentRow(lastNum);
 				return;
 			}
 
@@ -892,7 +959,35 @@ void MainWindow::importInstrumentsFromBank()
 								ui->instrumentListWidget, n,
 								QString::fromUtf8(name.c_str(), static_cast<int>(name.length())),
 								inst->getSoundSource(), instForms_));
+			lastNum = n;
 		}
+		ui->instrumentListWidget->setCurrentRow(lastNum);
+	}
+	catch (std::exception& e) {
+		QMessageBox::critical(this, tr("Error"), e.what());
+	}
+}
+
+void MainWindow::exportInstrumentsToBank()
+{
+	std::shared_ptr<BtBank> bank(std::make_shared<BtBank>(bt_->getInstrumentIndices(), bt_->getInstrumentNames()));
+
+	InstrumentSelectionDialog dlg(*bank, tr("Select instruments to save:"), this);
+	if (dlg.exec() != QDialog::Accepted)
+		return;
+
+	QString dir = QString::fromStdString(config_.lock()->getWorkingDirectory());
+	QString file = QFileDialog::getSaveFileName(this, tr("Save bank"), (dir.isEmpty() ? "./" : dir),
+												tr("BambooTracker bank file (*.btb)"));
+	if (file.isNull()) return;
+
+	std::vector<size_t> selection = dlg.currentInstrumentSelection().toStdVector();
+	std::sort(selection.begin(), selection.end());
+	if (selection.empty()) return;
+
+	try {
+		bt_->exportInstruments(file.toStdString(), selection);
+		config_.lock()->setWorkingDirectory(QFileInfo(file).dir().path().toStdString());
 	}
 	catch (std::exception& e) {
 		QMessageBox::critical(this, tr("Error"), e.what());
@@ -1221,6 +1316,15 @@ QString MainWindow::getModuleFileBaseName() const
 	return (filePath.isEmpty() ? tr("Untitled") : QFileInfo(filePath).completeBaseName());
 }
 
+int MainWindow::getSelectedFileFilter(QString& file, QStringList& filters) const
+{
+	QRegularExpression re(R"(\(\*\.(.+)\))");
+	QString ex = QFileInfo(file).suffix();
+	for (int i = 0; i < filters.size(); ++i)
+		if (ex == re.match(filters[i]).captured(1)) return i;
+	return -1;
+}
+
 /******************************/
 /********** Instrument list events **********/
 void MainWindow::on_instrumentListWidget_customContextMenuRequested(const QPoint &pos)
@@ -1257,6 +1361,8 @@ void MainWindow::on_instrumentListWidget_customContextMenuRequested(const QPoint
 	menu.addSeparator();
 	QAction* ldBank = menu.addAction(tr("&Import from bank file..."));
 	QObject::connect(ldBank, &QAction::triggered, this, &MainWindow::importInstrumentsFromBank);
+	QAction* exBank = menu.addAction(tr("E&xport to bank file..."));
+	QObject::connect(exBank, &QAction::triggered, this, &MainWindow::exportInstrumentsToBank);
 	menu.addSeparator();
 	QAction* edit = menu.addAction(tr("&Edit..."));
 	edit->setIcon(QIcon(":/icon/edit_inst"));
@@ -2347,3 +2453,8 @@ void MainWindow::on_actionShortcuts_triggered()
 	if (shortcutsDiag_->isVisible()) shortcutsDiag_->activateWindow();
 	else shortcutsDiag_->show();
 }
+
+void MainWindow::on_actionExport_To_Bank_File_triggered()
+{
+	exportInstrumentsToBank();
+}
