@@ -82,8 +82,11 @@ private:
 	void cloneInstrument();
 	void deepCloneInstrument();
 	void loadInstrument();
+	void funcLoadInstrument(QString file);
 	void saveInstrument();
 	void importInstrumentsFromBank();
+	void funcImportInstrumentsFromBank(QString file);
+	void exportInstrumentsToBank();
 
 	// Undo-Redo
 	void undo();
@@ -120,6 +123,7 @@ private:
 	void setModifiedTrue();
 	void setInitialSelectedInstrument();
 	QString getModuleFileBaseName() const;
+	int getSelectedFileFilter(QString& file, QStringList& filters) const;
 
 	bool isEditedPattern_, isEditedOrder_, isEditedInstList_;
 	bool isSelectedPO_;
@@ -227,6 +231,7 @@ private slots:
 	void updateVisuals();
 	void on_action_Effect_List_triggered();
 	void on_actionShortcuts_triggered();
+	void on_actionExport_To_Bank_File_triggered();
 
 	inline bool showUndoResetWarningDialog(QString text)
 	{
