@@ -563,6 +563,15 @@ std::string InstrumentsManager::getInstrumentName(int instNum) const
 	return insts_.at(static_cast<size_t>(instNum))->getName();
 }
 
+std::vector<std::string> InstrumentsManager::getInstrumentNameList() const
+{
+	std::vector<std::string> names;
+	for (auto& inst : insts_) {
+		if (inst) names.push_back(inst->getName());
+	}
+	return names;
+}
+
 std::vector<int> InstrumentsManager::getEntriedInstrumentIndices() const
 {
 	std::vector<int> idcs;
@@ -837,7 +846,7 @@ std::vector<int> InstrumentsManager::getOperatorSequenceFMEntriedIndices(FMEnvel
 	std::vector<int> idcs;
 	int n = 0;
 	for (auto& seq : opSeqFM_.at(param)) {
-		if (seq->isEdited()) idcs.push_back(n);
+		if (seq->isUserInstrument() || seq->isEdited()) idcs.push_back(n);
 		++n;
 	}
 	return idcs;
@@ -955,7 +964,7 @@ std::vector<int> InstrumentsManager::getArpeggioFMEntriedIndices() const
 	std::vector<int> idcs;
 	int n = 0;
 	for (auto& arp : arpFM_) {
-		if (arp->isEdited()) idcs.push_back(n);
+		if (arp->isUserInstrument() || arp->isEdited()) idcs.push_back(n);
 		++n;
 	}
 	return idcs;
@@ -1078,7 +1087,7 @@ std::vector<int> InstrumentsManager::getPitchFMEntriedIndices() const
 	std::vector<int> idcs;
 	int n = 0;
 	for (auto& pt : ptFM_) {
-		if (pt->isEdited()) idcs.push_back(n);
+		if (pt->isUserInstrument() || pt->isEdited()) idcs.push_back(n);
 		++n;
 	}
 	return idcs;
@@ -1187,7 +1196,7 @@ std::vector<int> InstrumentsManager::getWaveFormSSGEntriedIndices() const
 	std::vector<int> idcs;
 	int n = 0;
 	for (auto& wf : wfSSG_) {
-		if (wf->isEdited()) idcs.push_back(n);
+		if (wf->isUserInstrument() || wf->isEdited()) idcs.push_back(n);
 		++n;
 	}
 	return idcs;
@@ -1295,7 +1304,7 @@ std::vector<int> InstrumentsManager::getToneNoiseSSGEntriedIndices() const
 	std::vector<int> idcs;
 	int n = 0;
 	for (auto& tn : tnSSG_) {
-		if (tn->isEdited()) idcs.push_back(n);
+		if (tn->isUserInstrument() || tn->isEdited()) idcs.push_back(n);
 		++n;
 	}
 	return idcs;
@@ -1403,7 +1412,7 @@ std::vector<int> InstrumentsManager::getEnvelopeSSGEntriedIndices() const
 	std::vector<int> idcs;
 	int n = 0;
 	for (auto& env : envSSG_) {
-		if (env->isEdited()) idcs.push_back(n);
+		if (env->isUserInstrument() || env->isEdited()) idcs.push_back(n);
 		++n;
 	}
 	return idcs;
@@ -1521,7 +1530,7 @@ std::vector<int> InstrumentsManager::getArpeggioSSGEntriedIndices() const
 	std::vector<int> idcs;
 	int n = 0;
 	for (auto& arp : arpSSG_) {
-		if (arp->isEdited()) idcs.push_back(n);
+		if (arp->isUserInstrument() || arp->isEdited()) idcs.push_back(n);
 		++n;
 	}
 	return idcs;
@@ -1639,7 +1648,7 @@ std::vector<int> InstrumentsManager::getPitchSSGEntriedIndices() const
 	std::vector<int> idcs;
 	int n = 0;
 	for (auto& pt : ptSSG_) {
-		if (pt->isEdited()) idcs.push_back(n);
+		if (pt->isUserInstrument() || pt->isEdited()) idcs.push_back(n);
 		++n;
 	}
 	return idcs;
