@@ -1,12 +1,13 @@
 #include "jam_manager.hpp"
 #include <algorithm>
+#include <functional>
 #include <stdexcept>
 
-JamManager::JamManager(SongType type)
+JamManager::JamManager()
 	: isJamMode_(true),
 	  isPoly_(true)
 {
-	clear(type);
+	clear();
 }
 
 bool JamManager::toggleJamMode()
@@ -20,16 +21,16 @@ bool JamManager::isJamMode() const
 	return isJamMode_;
 }
 
-void JamManager::polyphonic(bool flag, SongType type)
+void JamManager::polyphonic(bool flag)
 {
 	isPoly_ = flag;
-	clear(type);
+	clear();
 }
 
-std::vector<JamKeyData> JamManager::keyOn(JamKey key, int channel, SoundSource source)
+std::vector<JamKeyData> JamManager::keyOn(JamKey key, int channel, SoundSource source, int keyNum)
 {
 	std::vector<JamKeyData> keyDataList;
-	JamKeyData onData{key, channel, source};
+	JamKeyData onData{ key, channel, source, keyNum };
 
 	std::deque<int>* unusedCh = nullptr;
 	switch (source) {
@@ -39,15 +40,36 @@ std::vector<JamKeyData> JamManager::keyOn(JamKey key, int channel, SoundSource s
 	}
 
 	if (!unusedCh->empty()) {
-		if (isPoly_) onData.channelInSource = unusedCh->front();
-		unusedCh->pop_front();
-		keyOnTable_.push_back(onData);
-		keyDataList.push_back(onData);
+		if (key == JamKey::MIDI_KEY) {
+			if (isPoly_) onData.channelInSource = unusedCh->front();
+			unusedCh->pop_front();
+			keyOnTable_.push_back(onData);
+			keyDataList.push_back(onData);
+		}
+		else {
+			auto&& it = std::find_if(keyOnTable_.begin(),
+									 keyOnTable_.end(),
+									 [&](JamKeyData x) { return (x.source == source && x.key == key); });
+			if (it == keyOnTable_.end()) {
+				if (isPoly_) onData.channelInSource = unusedCh->front();
+				unusedCh->pop_front();
+				keyOnTable_.push_back(onData);
+				keyDataList.push_back(onData);
+			}
+			else {
+				JamKeyData del = *it;
+				if (isPoly_) onData.channelInSource = del.channelInSource;
+				keyDataList.push_back(onData);
+				keyDataList.push_back(del);
+				keyOnTable_.erase(it);
+				keyOnTable_.push_back(onData);
+			}
+		}
 	}
 	else {
 		auto&& it = std::find_if(keyOnTable_.begin(),
 								 keyOnTable_.end(),
-								 [&](JamKeyData x) {return (x.source == source);});
+								 [&](JamKeyData x) { return (x.source == source); });
 		JamKeyData del = *it;
 		if (isPoly_) onData.channelInSource = del.channelInSource;
 		keyDataList.push_back(onData);
@@ -59,13 +81,14 @@ std::vector<JamKeyData> JamManager::keyOn(JamKey key, int channel, SoundSource s
 	return keyDataList;
 }
 
-JamKeyData JamManager::keyOff(JamKey key)
+JamKeyData JamManager::keyOff(JamKey key, int keyNum)
 {
 	JamKeyData keyData;
 
-	auto&& it = std::find_if(keyOnTable_.begin(),
-							 keyOnTable_.end(),
-							 [&](JamKeyData x) {return (x.key == key);});
+	auto cond = (key == JamKey::MIDI_KEY)
+			  ? std::function<bool(JamKeyData)>([&](JamKeyData x) -> bool { return (x.key == JamKey::MIDI_KEY && x.keyNum == keyNum); })
+			  : std::function<bool(JamKeyData)>([&](JamKeyData x) -> bool { return (x.key == key); });
+	auto&& it = std::find_if(keyOnTable_.begin(), keyOnTable_.end(), cond);
 	if (it == keyOnTable_.end()) {
 		keyData.channelInSource = -1;	// Already released
 	}
@@ -118,7 +141,26 @@ Note JamManager::jamKeyToNote(JamKey &key)
 	case JamKey::HIGH_AS:	return Note::AS;
 	case JamKey::LOW_B:
 	case JamKey::HIGH_B:	return Note::B;
-	default:	throw std::invalid_argument("Unexpected JamKey vlaue.");
+	default:	throw std::invalid_argument("Unexpected JamKey.");
+	}
+}
+
+JamKey JamManager::noteToJamKey(Note& note)
+{
+	switch (note) {
+	case Note::C:	return JamKey::LOW_C;
+	case Note::CS:	return JamKey::LOW_CS;
+	case Note::D:	return JamKey::LOW_D;
+	case Note::DS:	return JamKey::LOW_DS;
+	case Note::E:	return JamKey::LOW_E;
+	case Note::F:	return JamKey::LOW_F;
+	case Note::FS:	return JamKey::LOW_FS;
+	case Note::G:	return JamKey::LOW_G;
+	case Note::GS:	return JamKey::LOW_GS;
+	case Note::A:	return JamKey::LOW_A;
+	case Note::AS:	return JamKey::LOW_AS;
+	case Note::B:	return JamKey::LOW_B;
+	default:	throw std::invalid_argument("Unexpected Note.");
 	}
 }
 
@@ -155,22 +197,15 @@ int JamManager::calcOctave(int baseOctave, JamKey &key)
 	case JamKey::HIGH_C_H:
 	case JamKey::HIGH_CS_H:
 	case JamKey::HIGH_D_H:	return (baseOctave + 2);
-	default:	throw std::invalid_argument("Unexpected JamKey vlaue.");
+	default:	throw std::invalid_argument("Unexpected JamKey.");
 	}
 }
 
-void JamManager::clear(SongType type)
+void JamManager::clear()
 {
 	if (isPoly_) {
-		switch (type) {
-		case SongType::STD:
-			unusedChFM_ = std::deque<int>(6);
-			unusedChSSG_= std::deque<int>(3);
-			break;
-		case SongType::FMEX:
-			// UNDONE: change channel size by Effect mode
-			break;
-		}
+		unusedChFM_ = std::deque<int>(6);
+		unusedChSSG_= std::deque<int>(3);
 
 		for (size_t i = 0; i < unusedChFM_.size(); ++i) {
 			unusedChFM_[i] = static_cast<int>(i);
