@@ -11,7 +11,9 @@
 
 enum class SoundSource : int
 {
-	FM, SSG, DRUM
+	FM = 1,
+	SSG = 2,
+	Drum = 4
 };
 
 enum class SongType : int
@@ -45,6 +47,23 @@ enum class FMEnvelopeTextType : int
 	AR4, DR4, SR4, RR4, SL4, TL4, KS4, ML4, DT4
 };
 
+enum class FMOperatorType : int
+{
+	All, Op1, Op2, Op3, Op4
+};
+
+enum class SSGWaveFormType : int
+{
+	UNSET = -1,
+	SQUARE = 0,
+	TRIANGLE = 1,
+	SAW = 2,
+	INVSAW = 3,
+	SQM_TRIANGLE = 4,
+	SQM_SAW = 5,
+	SQM_INVSAW = 6
+};
+
 DECL_MAYBE_UNUSED
 static std::pair<int, Note> noteNumberToOctaveAndNote(int num)
 {
@@ -120,11 +139,11 @@ static int ctohex(const char c)
 DECL_MAYBE_UNUSED
 static uint8_t uitobcd(const uint8_t v)
 {
-	if (v > 99) throw std::out_of_range("out of range");
+	if (v > 99) throw std::out_of_range("Out of range.");
 
 	uint8_t high = v / 10;
 	uint8_t low = v % 10;
-	return (high << 4) + low;
+	return static_cast<uint8_t>(high << 4) + low;
 }
 
 DECL_MAYBE_UNUSED
@@ -134,3 +153,41 @@ static uint8_t bcdtoui(const uint8_t v)
 	uint8_t low = v & 0x0f;
 	return high * 10 + low;
 }
+
+DECL_MAYBE_UNUSED
+inline static size_t getFMChannelCount(SongType type)
+{
+	switch (type) {
+	case SongType::STD:		return 6;
+	case SongType::FMEX:	return 9;
+	default:	throw std::invalid_argument("Invalid SongType.");
+	}
+}
+
+DECL_MAYBE_UNUSED
+inline static bool isModulatedWaveFormSSG(SSGWaveFormType type)
+{
+	switch (type) {
+	case SSGWaveFormType::SQUARE:
+	case SSGWaveFormType::TRIANGLE:
+	case SSGWaveFormType::SAW:
+	case SSGWaveFormType::INVSAW:
+		return false;
+	case SSGWaveFormType::SQM_TRIANGLE:
+	case SSGWaveFormType::SQM_SAW:
+	case SSGWaveFormType::SQM_INVSAW:
+		return true;
+	default:
+		throw std::invalid_argument("Invalid SSGWaveFormType");
+	}
+}
+
+DECL_MAYBE_UNUSED
+inline static bool isModulatedWaveFormSSG(int id)
+{
+	try {
+		return isModulatedWaveFormSSG(static_cast<SSGWaveFormType>(id));
+	} catch (...) {
+		throw std::invalid_argument("Invalid id");
+	}
+}
