@@ -16,6 +16,9 @@ public:
 	static uint32_t ofInstrumentFileInBCD();
 	static std::string ofInstrumentFileInString();
 
+	static uint32_t ofBankFileInBCD();
+	static std::string ofBankFileInString();
+
 	static uint32_t toBCD(unsigned int major, unsigned int minor, unsigned int revision);
 	static std::string toString(unsigned int major, unsigned int minor, unsigned int revision);
 
@@ -23,7 +26,7 @@ private:
 	// Application version
 	static constexpr unsigned int appMajor			= 0;
 	static constexpr unsigned int appMinor			= 2;
-	static constexpr unsigned int appRevision		= 2;
+	static constexpr unsigned int appRevision		= 3;
 
 	// Module file version
 	static constexpr unsigned int modFileMajor		= 1;
@@ -35,6 +38,11 @@ private:
 	static constexpr unsigned int instFileMinor		= 2;
 	static constexpr unsigned int instFileRevision	= 1;
 
+	// Bank file version
+	static constexpr unsigned int bankFileMajor		= 1;
+	static constexpr unsigned int bankFileMinor		= 0;
+	static constexpr unsigned int bankFileRevision	= 0;
+
 	Version() {}
 };
 
@@ -68,6 +76,16 @@ inline std::string Version::ofInstrumentFileInString()
 	return toString(instFileMajor, instFileMinor, instFileRevision);
 }
 
+inline uint32_t Version::ofBankFileInBCD()
+{
+	return toBCD(bankFileMajor, bankFileMinor, bankFileRevision);
+}
+
+inline std::string Version::ofBankFileInString()
+{
+	return toString(bankFileMajor, bankFileMinor, bankFileRevision);
+}
+
 inline uint32_t Version::toBCD(unsigned int major, unsigned int minor, unsigned int revision)
 {
 	uint32_t maj = uitobcd(static_cast<uint8_t>(major));
